% REACTO(struc): Estimates central, low and high values of each carbon
%  type in each temperature step.
% struc is the OCEC object created by ReadOCEC.
% This program relies on Step2, which does most of the work. If called directly, 
%  Step2 returns a structure containing detail of the assumptions at each step.
function [datmid, datlo, dathi] = REACTO(os)
global datmid dathi datlo
global occ ocv pc lac bgs

% Set this to false to see the gory detail
isquiet = true;

% Calculate the data
[datmid, astru] = Step2(os, 'quiet', isquiet);
datlo = Step2(os, 'type','lo', 'quiet', isquiet);
dathi = Step2(os, 'type','hi', 'quiet', isquiet);

% Species to plot (these indices are defined in Step2)
ispec = [ocv lac pc];
isums = {[ocv occ], lac, pc};
tspec = {'OC ', 'LAC', 'PC'};

% Summarize data
cmap = [1 0.75 0; 0 0 1; 0.65 0.35 0.45;  0.375  1 0.375];   % color map

% Mode names
imodes = [1 2];
tmodes = {'Helium', 'Helium-Oxygen'};

% data for bar
Nsteps = length(datmid);

% Create new figure, and plot the bar graph representing the central guess
figure;
fsize = 16;
bgs=bar(datmid(:,ispec));

% Get one of the graphs for information
ch = get(gca, 'Children');
ch = ch(1);

% Data for error bars
x = 1:Nsteps;

% Set bar width. 
% I'm not sure this is the correct calculation, but it works.
xdel = get(ch,'Barwidth')/length(ispec) * 0.9;   % width of each bar
xbase = -xdel;                             % x-position to start

% Change the figure's size
figsize = get(gcf,'Position');
set(gcf, 'Position', figsize .* [1 1 1.5 1]);       % stretch the figure
set(gcf, 'Name', ['REACTO/' os.name]);
set(gca, 'XLim', [0.5 (max(x)+0.5)]);
hold on

% plot error bars (ispec loops through species)
for j=1:length(ispec)
  i=ispec(j);
  eb=errorbar(x+xbase, datmid(:,i), (datmid(:,i)-datlo(:,i)), dathi(:, i)- datmid(:,i));
  set(eb, 'LineStyle', 'none', 'Marker', 'none');
  xbase = xbase + xdel;
  
end

% Calculate totals
for j=1:length(ispec)
   icidx = isums{j};
   sums = [0 0 0];
   
   for k=1:length(icidx)
      pt = icidx(k);
      sums = sums + [sum(datlo(:, pt)) sum(datmid(:, pt)) sum(dathi(:, pt))];      
   end
   
  % add to legend
  cent = sums(2);
  uncert = max(abs(sums-cent));
    tspec{j} = [tspec{j} ' ' num2str(cent, '%5.2f') '+/-' ...
      num2str(uncert, '%5.2f')];
   
end

% Dress up figure
set(gca, 'FontSize', 14);
legend(tspec);

% Create x-label (step# within analysis period)
tlabl = {};
modelist = [];
lastmode = -99;
for i=1:size(astru.mode)
  stepi = find(astru.fracid==i, 1);
  if astru.mode(stepi) ~= lastmode
      n=1;
      lastmode = astru.mode(stepi);
  end
  tlabl{i} = num2str(n);
  modelist(i) = lastmode;
  n=n+1;
end

% Apply tick label to graph
set(gca, 'XTickLabel', tlabl);

% Dress up graph
colormap(cmap);
set(gca, 'FontSize', fsize);
tl = title(os.name);
set(tl, 'Interpreter', 'none');

% Axis limits
xlim = get(gca, 'XLim');
ylim = get(gca, 'YLim');
ylim(1) = min(ylim(1), -1);
set(gca, 'YLim', ylim);

% Add mode markers
yloc = ylim(1) + 0.08 * diff(ylim);
for i=1:length(imodes)
    spl = find(modelist==i, 1);

    % Divide modes
    if i>1
       plot([spl spl]-0.5, ylim, ':');
    end

    text(spl-0.25, yloc, tmodes{i}, 'FontSize', fsize);
end

% Add labels
ylabel('Loading (ug/cm2)');
tx=text(xlim(2)+0.5, ylim(2)/2, 'Release', 'FontSize', fsize, 'Rotation', 90);
tx=text(xlim(2)+0.5, ylim(1)/2, 'Formation', 'FontSize', fsize, 'Rotation', 90, 'HorizontalAlignment', 'center');

return
