% ReadOCEC(filename, testID, rawdir, params) produces a structure based
%  on the raw data file, with signals translated into physical terms. 
% Required arguments:
%   filename     name of raw data file (including extension)
%   testID       test label (must be exact, including spaces)
%   rawdir       location of raw data files
% Optional arguments:
%   params       structure with fields Cal_Loop_Carbon, Filter_Area, 
%                  Transit_Time; if it is passed in, program assumes 
%                  that raw file has older structure
% Returns:
%   datacell     structure full of vectors & parameters, listed below.
% Structure fields:
%   name-testID; info-blank; CalibLoopCarbon; FilterPunch-filter area(cm2); 
%   TransitTime; transfit-polynomial fit for blank laser; 
%   split_tr-split time for transmittance (sec)
% Vectors in structure (all divided into separate analysis & _cal periods)
%   time-seconds; temp-temperature(C); FID-carbon signal(ug/cm2); 
%   carb_left-carbon remaining on filter (ug/cm2); zeroFID-baseline;
%   mode-1/He,2/HeOx,3/cal; laser-laser signal (arbitrary); 
%   abs-ATN/100; cabs-ATN corrected for filter temp dependence;
%   blaser-assumed blank filter ATN

function dstruc = ReadOCEC(filename, testID, rawdir, params)

% Author: Tami Bond, University of Illinois, yark@uiuc.edu
% Version 2.1    Last update: June 2006
% This program is distributed under the Creative Commons by-sa license 3.0
%   (http://creativecommons.org/licenses/by-sa/3.0/).
%
% Required additional files: ReadRawData, smfilt
% Functions embedded here: GetSplitTime
%
% ----------------------- Program begins --------------------------------
% interpret inputs
if nargin<2
    help ReadOCEC;
    return
end
if nargin<3
    rawdir='';
end
isold = (nargin>=4);
global FIDLo_off FIDHi_off

% Working parameters
Temp_Calm = 0.0299;          % Cal for temperature sensor
Temp_Calb = -8;

Temp_Limit = 60;           % Temp assuming zero carbon release (for FID cal)

FID_Lo_Limit = 2;           % Corrected FID signal that represents limit
                            % of low measurement range
                            
% Get data. Columns are:
% 1-FID1; 2-FID2; 3-front_temperature; 4-laser; 5-laserlow; 
% 6-pressure; 8-mode
% Modes are: 1-He; 2-Ox; 3-CH4+Ox(cal); 0-undefined
if length(strfind(filename, '.'))==0            % no extension given
    filename = [filename '.txt'];
end

% Acquire the raw data using ReadRawData program. The last argument
%  is passed in if file is in the old format
if isold
   datamat =...
       ReadRawData(filename, testID, rawdir, '\t', params.Transit_Time);
else
   [datamat, params] = ReadRawData(filename, testID, rawdir);
end

ndat = size(datamat, 1);
if ndat==0       %nothing returned
    dstruc=struct();
    return;
end

% Index definitions (to use for readability)
idx_fidlo = 1;
idx_fidhi = 2;
idx_temp = 3;
idx_lsrhi = 4;
idx_lsrlo = 5;
idx_refhi = 7;
idx_reflo = 8;
cal_mode = 3;       % mode number for calibration

idx_mode = size(datamat, 2);    % assume mode is in last column
hasrefl = size(datamat, 2)>8;   % reflectance present 

% calibration times
caltimes = find(datamat(:, idx_mode)==cal_mode);

% Correct temperatures
Temps = (Temp_Calm * datamat(:, idx_temp)) - Temp_Calb;

% ----- Calculate FID offset-- the signal when there is no carbon ---------
% Obtain FID zero
% Index before which there is no temp rise -- TOO CONSERVATIVE, may want to
% change??
fid0_beg = [];
while isempty(fid0_beg)
   fid0_beg = (1:(max(find(Temps<Temp_Limit)-params.Transit_Time)))';
   Temp_Limit = Temp_Limit + 5;     % notch up 5 degrees
   if Temp_Limit>100
       disp('Warning! Initial temperature over 100C');
   end
end

% Directions for finding end FID offset
numnegsok = 5;
nonzerocalfrac = 0.005;

zeroatcalperiod = false;        % marker: is zero from end of cal?

% Search for post-test FID zero. Differential representing zero change:
FID_Diff_Allow=[-2 0];           

% Look around the beginning of the calibration period
zsearchidx = (caltimes(1) - 20):(caltimes(1) + 5);

% These periods correspond to zero change in FID
diff_fid = [0; diff(datamat(:, idx_fidlo))];
fid0_end = find(diff_fid(zsearchidx)>=FID_Diff_Allow(1) & ...
    diff_fid(zsearchidx) <= FID_Diff_Allow(2)) + ...
    (zsearchidx(1) - 1);

% found a zero at the end of the analysis period: get and evaluate offset 
if ~isempty(fid0_end)
   FIDLo_off = fid_offset(datamat(:, idx_fidlo), fid0_beg, fid0_end, caltimes, false);

   % FIDLo offset can lead to negative concentrations, because fid0_end may
   % contain a tail of still-evolving carbon. If so, re-do the offset calc. 
   netfid = (datamat(:, idx_fidlo)- FIDLo_off);
   negdef = -mean(netfid(caltimes))*nonzerocalfrac;

% No zero period at end of analysis; use a dummy difference
else
   netfid = [];
   negdef = 0;
end

% Too many negatives, OR no zero period found: try end of cal period
if isempty(fid0_end) | length(find(netfid(1:caltimes(1)) < negdef)) > numnegsok;
   zeroatcalperiod = true;
   fid0_end = ((ndat-9):ndat)';
   FIDLo_off = fid_offset(datamat(:, idx_fidlo), fid0_beg, fid0_end, caltimes, true);
end

% Check for negatives again
netfid = (datamat(:, idx_fidlo)- FIDLo_off);
negdef = -mean(netfid(caltimes))*nonzerocalfrac;

% Still too many negatives: just use beginning
if length(find(netfid(1:caltimes(1)) < negdef)) > numnegsok;
    zeroatcalperiod = false;
    fid0_end = fid0_beg;
   FIDLo_off = fid_offset(datamat(:, idx_fidlo), fid0_beg, fid0_end, caltimes, false);
end

% Get FIDHi offset, based on the chosen times
FIDHi_off = fid_offset(datamat(:, idx_fidhi), fid0_beg, fid0_end, caltimes, zeroatcalperiod);

% These commands will plot the background data if uncommented
%figure;
%plot(1:ndat, FIDLo_off, 'b', 1:ndat, datamat(:, idx_fidlo), 'g')
%set(gca, 'YLim', [-500 300]);
%xline([min(fid0_end) max(fid0_end)]);

%zeroPoints = [zeroTmp; (ndat-zeroCal:ndat)'];
zeroPoints = [fid0_beg; fid0_end];

% Calculate FID calibration. Units of the corrected trace will be 
% ug/sec; that is, if you integrate over dt, you should get ug.
% This calibration works differently than the temp calibration,
% and the offset is subtracted before multiplying by the slope.
FIDLo_cal = params.Cal_Loop_Carbon/...
    sum(datamat(caltimes, idx_fidlo) - FIDLo_off(caltimes));
FIDHi_cal = params.Cal_Loop_Carbon/...
    sum(datamat(caltimes, idx_fidhi) - FIDHi_off(caltimes));

% Calculate FID low and high signals, and combine them to get the best 
% signal. 
FIDHi = (datamat(:, idx_fidhi)- FIDHi_off) * FIDHi_cal;
FIDSig = (datamat(:, idx_fidlo)- FIDLo_off) * FIDLo_cal;

% Find FID signals that are in danger of being saturated & replace them. 
fiddanger = find(FIDSig>FID_Lo_Limit  &...
            datamat(:, idx_mode~=3) & abs(FIDHi-FIDSig)>0.05);
FIDSig(fiddanger) = FIDHi(fiddanger);

% Calculate amount of carbon left on filter at each time step
workfid = FIDSig;
workfid(caltimes)=0;            % don't count cal loop carbon
FID_left = cumsum(workfid(ndat:-1:1));      % integrate backwards
FID_left = FID_left(ndat:-1:1);          % switch order of loop

% Correct laser signal, assuming it is clean during cal loop (wrong!!)
%Laser_I0 = mean(datamat(caltimes, idx_lsrhi));

% Temp increased or decreased since last step?
dT = [0; diff(Temps)];

% Calculate & smooth using a median filter
nmedpoints = 11;
LaserSig = smfilt(datamat(:, idx_lsrhi), nmedpoints);
ReflSig = smfilt(datamat(:, idx_refhi), nmedpoints);

% separate calibration from non-cal
modelist = datamat(:, idx_mode);
noncal = find(modelist~=cal_mode);
cal = find(modelist==cal_mode);

% estimate transmission through blank filter
trans_fit_ord = 2;                                  % order of transmission fit
min_temp_fit = 200;                            % don't use fit below this temperature (C)
pf = polyfit(Temps(cal), LaserSig(cal), trans_fit_ord);   % poly fit during cal time
Trans_est = polyval(pf, Temps);

% estimate transmission at 'low' temps (use 20-deg bound above min_temp_fit)
lowT_trans_est = mean(Trans_est(find(Temps>min_temp_fit & Temps<min_temp_fit + 20)));
Trans_est(find(Temps < min_temp_fit)) = lowT_trans_est;

% Calculate I/I0 = abs carbon released or formed
nmedpoints = 9;
deltaAbs = smfilt([0; diff(log(LaserSig ))], nmedpoints);

% Trans_est 
deltaAbscorr = smfilt([0; diff(log( LaserSig ./ Trans_est))], nmedpoints);

% estimate transmission at 'low' temps (use 20-deg bound above min_temp_fit)
lowT_trans_est = mean(Trans_est(find(Temps>min_temp_fit & Temps<min_temp_fit + 20)));
Trans_est(find(Temps < min_temp_fit)) = lowT_trans_est;

% determine split point for transmittance trace- per D. Smith
[split_tr, corr_tr] = SplitTime(LaserSig, cal(1));

if hasrefl
   [split_rf, corr_rf] = SplitTime(ReflSig, cal(1));
end

% put data in structure
dstruc = struct('file', filename, 'name', testID, 'info', '',...
    'CalibLoopCarbon', params.Cal_Loop_Carbon,...
    'FilterPunch', params.Filter_Area,...
    'TransitTime', params.Transit_Time,...
    'time', noncal, 'time_cal', cal,...
    'temp', Temps(noncal), 'temp_cal', Temps(cal),...
    'FID', FIDSig(noncal)/params.Filter_Area,...
    'FID_cal', FIDSig(cal)/params.Filter_Area,...
    'carb_left', FID_left(noncal)/params.Filter_Area,...
    'zeroFID', zeroPoints, 'mode', modelist,...
    'laser', LaserSig(noncal), 'laser_cal', LaserSig(cal),...
    'ATN', deltaAbs(noncal)*100, 'ATN_cal', deltaAbs(cal),...
    'cATN', deltaAbscorr(noncal)*100, 'cATN_cal', deltaAbscorr(cal),...
    'blaser', Trans_est(noncal), 'blaser_cal', Trans_est(cal),...
    'transfit', pf, 'istemprise', dT(noncal)>0, ...
    'laserlo', datamat(:, idx_lsrlo),...
    'split_tr', split_tr, 'lasercorr_tr', corr_tr);

% Add reflectance information
if hasrefl
   dstruc.refl = ReflSig(noncal);
   dstruc.refl_cal = ReflSig(cal);
   dstruc.split_rf = split_rf;
end

return

% Get split time-- algorithm given by D. Smith, Sunset
function [spidx, lasercorr] = SplitTime(laser, calstart)

% number of averaging points to use for laser start/end
nlcpts = 10;

% initial time to examine, to find max laser
initperd = 70;

npts = length(laser);

% Laser correction for temp dependence
lasercorr = mean(laser(calstart:(calstart+nlcpts-1)))...
    /mean(laser((npts-nlcpts+1):npts));

% initial transmittance-- find max in first 70 sec (Laser is already
% smoothed)
initlaser = max(laser(1:initperd));      

% value to look for
seeklaser = initlaser * lasercorr;

% search backward
spidx = find(laser(1:(calstart-1))<seeklaser, 1, 'last');

return

% --- Interpolate the FID offset ----------------
% Arguments: signal, index_of_beginning, index_of_end, index_of_cal, 
%   using_end_of_calperiod
function fidoff = fid_offset(fidsig, fid0_beg, fid0_end, caltimes, isfromcal)

intpt1 = max(fid0_beg)+1;
intpt2 = caltimes(1);

% First fixed point
fidoff(fid0_beg) = mean(fidsig(fid0_beg));

% Find last fixed point
% Using calibration: account for saturation
if isfromcal
   fidoff(caltimes) = findcal0(fidsig(caltimes));

% Just use points marked by fid0_end
else
   fidoff(caltimes) = mean(fidsig(fid0_end));
end

% Interpolate between the two end points
fidoff(intpt1:intpt2) = fidoff(1) + (fidoff(caltimes(1))-fidsig(fid0_beg(1))) *...
    (1:(intpt2-intpt1+1))/(intpt2-intpt1);
fidoff = fidoff';

return

% --- Find final value of cal period, considering FID saturation ----
function endzero = findcal0(y)

% From calibration experiments
satmult = 1.0007;

maxy = max(y);
leny = length(y);
decay1 = (leny-10):leny;

% peak height
pkht = maxy - mean(y(decay1));

% return zero value
endzero = maxy - (pkht * satmult);

return
