%ReadRawData(filename, testID, rawdir, delm, transit) 
%  produces the raw data array read from a file. 
% Required arguments:
%   filename     name of raw data file (including extension)
%   testID       test label (must be exact, including spaces)
%   rawdir       location of raw data files
% Optional arguments:
%   delm         delimiter in raw file (default tab)
%   transit      transit time; if given, program assumes 
%                  older file structure (no calconst, farea, transit
%                  recorded in file)
% Returns:
%   datamat      array
%   params       structure containing Cal_Loop_Carbon, Filter_Area, 
%                  Transit_Time (empty if transit time given is old)
% T.C. Bond, Jun 2004

function [datamat, params] = ...
    ReadRawData(filename, testID, rawdir, transit, delm)

% initialize variables
datamat=[];
params={};

if nargin<2
    help ReadRawData
    return
end
% default for optional arguments
if nargin<3
    rawdir = '';
end
if nargin<4
    isold = false;
else
    params.Transit_Time = transit;
end
if nargin<5
    delm = '\t';                     % delimiter (\t for tab)
end


% set up directory location
if nargin<3 
    dataloc = '';
elseif length(rawdir)==0
elseif ~strcmp(rawdir(length(rawdir)), '\')
    rawdir = [rawdir '\'];
end

% open file
fha = fopen([rawdir filename], 'r');         
if fha<0
    disp('File not found');
    return
end

TestFound = false;
testID = deblank(testID);

% Look through file
while (~TestFound) & (~feof(fha))
    
    % Find a sample
    if ~FindString1(fha, 'Sample')
        continue
    end
    
    % Check if this is right test number
    ftest=fgetl(fha);  
    
    if length(ftest)==0             % this is a kludge to deal with strange end-of-lines from unix etc
        ftest=fgetl(fha);
        skipline = true;            % determine that you need to advance line twice
    else
        skipline = false;
    end
    
    % Compare with test name
    if strcmp(deblank(ftest), testID)
       TestFound = true;
    end
        
end

% Get parameter datadata
params = struct();
if TestFound & ~isold
   FindString1(fha, 'CalConst', skipline);
   params.Cal_Loop_Carbon = str2num(deblank(fgetl(fha)));
   FindString1(fha, 'FilterArea', skipline);
   params.Filter_Area = str2num(deblank(fgetl(fha)));
   FindString1(fha, 'TransitTime', skipline);
   params.Transit_Time = str2num(deblank(fgetl(fha)));  
end

% Read the trace
if TestFound
    
    % Find the header
   [fs, hline] = FindString1(fha, {'FID1,', '"FID1,"', 'FID1'}, skipline);
   
   % Get variable string based on format
  evalstr = Str2Eval(hline, delm);
   
   % Get trace
   datamat = Read_Trace(fha, params.Transit_Time, evalstr, skipline);
else
   datamat=[];
end

fclose(fha);

return

% Read OC/EC trace data. 
% Parameters: fh=file handle; rhead = row header; tr_time=transit time;
% delm=delimiter; skipline=blank line between rows
function rmat=Read_Trace(fh, tr_time, evalstr, skipline)
rmat = [];
modemat = [];

 % Process file header

 % Loop through analysis lines until complete
 while true & ~feof(fh)
      
      r=fgetl(fh);          % file data
      
      if skipline           % need to skip extra line (old format)
          fgetl(fh);
      end

      % End of test record
      if size(r, 2)==0
          break;
      end
     
      % This parses the string based on the definition passed in
      eval(evalstr);
      
      cdat = cell2mat(cdat);
      % Find classification
      switch cdat
          case 'He'
              modeno=1;
          case 'Ox'
              modeno=2;
          case 'CH4+Ox'
              modeno=3;
          case 'CH4+He'
              modeno=3;
          otherwise
              modeno=0;
      end
          
     rmat = [rmat; ndat];
     modemat = [modemat; modeno];
 end

rmat = [rmat modemat];
% Move FID signals down to account for transit time
lendat = size(rmat, 1);
rmat(1:lendat-tr_time, 1) = rmat(tr_time+1:lendat, 1);
rmat(1:lendat-tr_time, 2) = rmat(tr_time+1:lendat, 2);
rmat(lendat-tr_time+1:lendat, 1) = rmat(lendat-tr_time, 1);
rmat(lendat-tr_time+1:lendat, 2) = rmat(lendat-tr_time, 2);
return

% Process a test header to determine the readin format. Assumes the string
% read in will be in variable r.
function evalstr = Str2Eval(hline, delm)

evalstr = '[';
formatstr = '';
commaloc = [0 findstr(hline, ',') length(hline)+1];

% numeric & character indices
in = 1;
ic = 1;

% loop through variable names
for i=1:length(commaloc)-1
    
   % get variable name
   varname = deblank(hline((commaloc(i)+1):(commaloc(i+1)-1)));
   while varname(1)==' '
       varname = varname(2:length(varname));
   end
       
   switch varname

       % mode variable (there's a numeric after it but I don't use it)
       case 'mode'
           evalstr = [evalstr 'cdat, '];
           formatstr = [formatstr '%s'];
           ic = ic+1;
                
       % numeric variable
       otherwise
           evalstr = [evalstr 'ndat(' num2str(in) '), '];
           formatstr = [formatstr '%d'];
           in = in+1;
   end

end

% the 'dummy' numeric
evalstr = [evalstr 'ndat(' num2str(in) ')]'];
formatstr = [formatstr '%d'];
in = in+1;
           
% put it all together
evalstr = [evalstr '= strread(r, ''' formatstr ''', ''delimiter'''...
  ', ''' delm ''');'];

return

% Find a string at a beginning of a line by scanning through file. 
% Crude but it works. Not recommended for large files.
function [StringFound, fline] = FindString1(fh, strtext, skipline)

if nargin<3
    skipline = false;
end
if ~iscell(strtext)
    strtext = {strtext};
end

nstrs = length(strtext);
strlen = size(strtext, 2);
for i=1:nstrs
    strlen(i)=size(cell2mat(strtext(i)), 2);
end

StringFound = false;
n=1;
while (~StringFound) & (~feof(fh))
    fline = fgetl(fh);
    for i=1:nstrs
        if strcmp(fline(1:min(size(fline, 2), strlen(i))), strtext(i))
           StringFound = true;
       end
    end
end

if skipline
   fgetl(fh);
end

return
