% Step2(struc, 'arg', argval): Backward stepping program to differentiate PC/LAC in TOA
% Required arguments: 
%    struc:     TOA structure obtained from ReadOCEC
% Optional arguments:
%    type:      analysis type- refers to estimate of LAC [ {med} | lo | hi ]
%    plot:      plot type [ {area} | line | none]
%    quiet:     run analysis quietly? [true | {false}]
% Returns: 
function [fs, aos, rmessage] = Step2(os, varargin)

% Author: Tami Bond, University of Illinois, yark@uiuc.edu
% Version 2.1    29 Nov 2007
% This program is distributed under the Creative Commons by-sa license 3.0
%   (http://creativecommons.org/licenses/by-sa/3.0/).
% 
% Step2 requires some additional files:
%   GetTempProgram, GetAtn (advanced users may attempt to modify)
%   Step_Assumption, Step_Underdet
%   varg_val, varg_proof (utilities)
% Functions embedded in this file:
%   StepKernel, GetTimeSteps, MakeSumData, FracSum, GetSign
%

% Species: OCc, OCv, OCh PC, LAC
global SpeciesN StepN tprog
global occ ocv och pc lac oxmode
global xdiff zerocarb

% --------------- Program begins -----------------------------------------
% Note all temperatures are in degC
%
% Settings (user may change)
maxiter = 100;                     % maximum number of iterations
lumplen = 10;                     % basic time period to analyze (sec)

min_ksa = 1.8;                     % minimum k-sigma for lac
max_ksa = 15;                     % maximum k-sigma for lac
ksa_init = 6.2;                   % initial guess of k-sigma for LAC

zeropc = 0.03;                    % value of zero PC, required for convergence
zerocarb = 0.02;                 % value of zero carbon, used to assess mismatch

fs = [];
rmessage = struct();

% Check inputs
if ~varg_proof(varargin, {'type', 'plot', 'quiet'}, true);
     return
end

% Process arguments
type = varg_val(varargin, 'type', 'med');
makeplot = varg_val(varargin, 'plot', 'default');
quiet = varg_val(varargin, 'quiet', false);

if strcmp(makeplot, 'default')
    if nargout==0
        makeplot = 'area';
    else
        makeplot = 'none';
    end
end
    
%Indices, for program readability. Since this is the main program, the
% definition is given here and picked up by QuantUncert if needed.
occ=1;
ocv=2;
lac=3;
pc=4;
labl = {'occ', 'ocv', 'lac', 'pc'};
oxmode = 2;

% ---- Identify the analysis steps and crunch the data to be analyzed 
% program length
proglen = length(os.temp);

% Get the time steps by providing the length of the temp. vector.
[tprog, fracid] = GetTempProgram(proglen);

% Check if temperature program was found. Temp program is only important
%  to make sure that each step contains an integral number of time lumps. 
% It will also be used in graph. 
if length(tprog)==0
  disp(['  no temp program']);
  return
end

% Get the time steps for analysis. 
[asteps,afrac] = GetTimeSteps(tprog, fracid, lumplen);

% Summarize the data in the analysis steps.
% This also adds a description of flags.
aos = MakeSumData(os, asteps, afrac);

StepN = max(fracid);
SpeciesN = 4;

% initialize zero vector
x = zeros(SpeciesN, 1);

% Start the K-sigma guesses. This will be iterated, so doesn't matter much.
ksguess = zeros(SpeciesN, 1);
ksguess(lac) = ksa_init;

% Set up for iteration
niter = 0;           % number of iterations
is_sing = 0;            % singularity experienced

% Constraints for nudging. 'Mismatch' here refers to mismatch between 
%   PC sign and LAC sign, indicative of the wrong type of transmittance
%   dependence curve.
nudgeval = 0.2;        % Amount to nudge LAC optics stdev if there is a mismatch
nudgedir = 1;          % Direction to nudge (changes during analysis)
maxnudge = 2;        % Maximum value of standard deviation for nudging
mismatchok = 2;        % number of mismatched points that are accepted
aos.mismatch(1:aos.N) = 1;    % Start with mismatch so iteration 1 will proceed
Nhists = 4;
mm_hist = aos.N * ones(1, Nhists);     % History of mismatches

% Define the underdetermined weights (must sum to one)
% The first vector contains the least LAC. 
uweight = [0.5; 0.5];
uweightdel = [0.1; -0.1];        % must sum to zero

% Amount to nudge guess of ksigma toward new estimated value 
% (0 is not at all; 1 is use new estimate completely. Low values
%  risk not converging; high values risk oscillation.)
ksnudge = 0.5;

% Turn warnings off (these are handled by the program)
oldwarns = warning('off', 'MATLAB:nearlySingularMatrix');

% Set z-values (std devations from mean for optical properties)
opticsz = zeros(1, SpeciesN);

% Change uweight, opticsz, and ksnudge for different cases
switch type        
   case 'lo'
       opticsz(lac) = 1;
       opticsz(pc) = -1;
       uweight = [0.1; 0.9];
       ksnudge = 0.1;
       
   case 'hi'
       opticsz(lac) = -1;
       opticsz(pc) = 1;
       uweight = [0.9; 0.1];
       ksnudge = 0.1;
end

if ~quiet
   disp(['----------------' os.name '------------------------']);
end

% --- Loop 1: Iterate until PC and LAC have same sign
while (sum(aos.mismatch) > mismatchok) & niter<maxiter
    
   % --- Loop 2: Iterate until zero PC is obtained 
   x(pc)=99;                % Start with really high value so iteration 2 will proceed
   while abs(x(pc))>zeropc & niter<maxiter
      niter = niter + 1;

      % Debug line (note that mismatch will be from previous data)
      if ~quiet
      disp(['iter' num2str(niter) ': mis' num2str(sum(aos.mismatch)) ...
       ', nudge: ' num2str(nudgedir) ', opticsz:' num2str(opticsz(lac)) ', ksa:' num2str(ksguess(lac))])
      end
   
      % Run the main program for one calculation
      aos = StepKernel(aos, x, ksguess, opticsz, uweight);
      x = aos.x(1,:);
      
      % Calculate the average ksigma for each species
      ksav = sum(aos.dx .* aos.ks) ./ sum(aos.dx);

      % Don't change the zero species
      zeroidx = find(abs(sum(aos.dx))<zerocarb);
      ksav(zeroidx) = ksguess(zeroidx);
   
      % New guess for LAC k-sigma-a, if there is some PC left over
      finalx = aos.x(1,:);

      if abs(finalx(lac))>zerocarb           % prevents singularity if lac=0
         newksav = (finalx(pc) * ksav(pc) + finalx(lac) * ksav(lac))/finalx(lac);
      else
         newksav = ksav(lac);
      end
      
      % nudge toward new guess
      newguess = ([newksav / ksav(lac) 1] * [ksnudge (1-ksnudge)]') * ksguess(lac);
      
      % --- Determine next actions based on k-sigma. The next command
      % sets the flag to 0 if newguess is in the right range, -1 if it
      % is below min_ksa, 1 if it's above max_ksa.
      % msg1 is a debugging command
      ksaflag = (find(newguess<[min_ksa max_ksa 9999], 1))-2;
      
      % Converged; don't change ksa
      if abs(x(pc)) < zeropc
          msg1 = ['ks1, converged at pc=' num2str(abs(x(pc)))];

      % Need a new guess, and prediction is fine
      elseif ksaflag==0
          msg1 = ['ks2,' num2str(newguess), ' pc=' num2str(x(pc))];
          ksguess(lac) = newguess;

      % Nudge the underdetermined weights
      elseif max(uweight - ksaflag * uweightdel) <=1
         uweight = uweight - ksaflag * uweightdel;
         msg1 = ['ks3, uweight = ' num2str(uweight')];
         
      % KSGuess is not fine, but it was a big change; try the minimum
      elseif ksguess(lac) - min_ksa > (min_ksa * 0.1)
          ksguess(lac) = min_ksa;
          msg1 = 'ks4, now min';

      % Nothing worked; try a new LAC curve
      else
          msg1 = ['ks5: breaking (newguess was ' num2str(newguess) ')'];
          break
      end               % End of k-sigma branching ---
      
      % Write out debug message
      if ~quiet
         disp(msg1);
      end
      
      % Check for badly scaled matrices
      [l1, l2] = lastwarn;
      if strcmp(l2, 'MATLAB:nearlySingularMatrix')
         is_sing = 1;
         break;
      end
      
   end    % ------ Done with PC iteration (2nd loop) ----------

   % Check whether PC and LAC have mismatched sign
   pcsign = GetSign(aos.dx(:,pc), zerocarb);
   lacsign = GetSign(aos.dx(:,lac), zerocarb);
   is_mismatch = (pcsign ~= lacsign & pcsign~=0 & lacsign~=0 & aos.mode==oxmode);

   % Add the number if mismatches to the history
   mm_hist = circshift(mm_hist, [0 1]);
   mm_hist(1) = sum(is_mismatch);
   
   % Check whether nudge can help
   % Improvement in mismatches: keep same direction
   %   msg2 describes the nudge result
   if max(diff(mm_hist))>0
      msg2 = 'nd1, nudging good';

   elseif nudgedir < 0          % already switched direction
      msg2 = ['nd2, giving up (now ' num2str(mm_hist) ')'];
      break

   else                         % Could try a new direction; start mm_hist over
      msg2 = ['nd3, switching direction: was ' ...
          num2str(sum(aos.mismatch)) ', now ' num2str(sum(is_mismatch))];
      nudgedir = -1;
      mm_hist = aos.N * ones(1, Nhists);     % History of mismatches
      
   end
   
   % Print debug message
   if ~quiet
   disp(msg2);
   end
   
   % nudged too much!! give up
   if abs(nudgeval) > maxnudge
       break
   end

   % Too many mismatches: keep trying
   if sum(is_mismatch) > mismatchok & (niter+1)<maxiter
       opticsz(lac) = opticsz(lac) + nudgeval * nudgedir;
       ksguess(lac) = ksa_init;
   end
       
   aos.mismatch = is_mismatch;
end

% --- Convey result ----------------------
msgr = [type ': '];
success = 1;
if is_sing
   msgr = [msgr 'resulted in singular matrix; '];
   success = 0;
end
if (abs(x(pc))>zeropc) 
   msgr = [msgr 'failed to converge; '];
   success = 0;
end
if (sum(aos.mismatch) > mismatchok)
   msgr = [msgr 'wrong sign of PC/LAC (' num2str(sum(aos.mismatch)) '),'];
   success = 0;
end
if sum(~aos.solfound)>0
   msgr = [msgr 'no solution at step ' num2str(find(~aos.solfound)') '; '];
   success = 0;
end
if success
    msgr = [msgr 'converged after ' num2str(niter) ' iterations; '];
end

msgr = [msgr 'ksa-lac now ' num2str(ksguess(lac))];
if ~quiet
disp(msgr);
end

% Fill in output structure
rmessage.message = msgr;
rmessage.success = success;
rmessage.iterations = niter;

% --- Show graph if output is required ---
if strcmp(makeplot, 'area')
   area(cumsum(aos.len), aos.dx);
   legend(labl, 'Location', 'Best');
elseif strcmp(makeplot, 'line')
   figure;
   hold on
   colrs = get(gca, 'ColorOrder');
   for i=1:SpeciesN
       plot(cumsum(aos.len), aos.dx(:,i), 'Color', colrs(i, :));      
   end
   legend(labl, 'Location', 'Best');
   hold off
   
end

% Summarize steps. fs is the change in each species during each step.
% It is a [Nsteps x Nspecies] matrix. 
fs = FracSum(aos);

% Set warnings back to original state
warning(oldwarns);

return

% -----------------------------------------------------------------------
% StepKernel is one calculation through the time series. It is called by 
%  the iteration loop.
%  osin is the structure containing test information (from ReadOCEC)
%  xold is the value of x determined in the previous iteration.
%  y is [FID;ATN]. opticsz is the number of standard deviations away from
%  the central value for each guess of optics. 
% Several flags are set in this subroutine. They are stored in osout,
%  which is a copy of osin plus the new flags. This can be useful for 
%  debugging or understanding the analysis at that point. 
function osout = StepKernel(osin, xold, ksguess, opticsz, uweight)
global SpeciesN 
global occ ocv lac pc oxmode
global rechout R stepy zerocarb
osout = osin;

% Define small changes: these have been tweaked to optimize
% Below these threshholds, inconsistencies are ignored, because they
%   are too small to worry about. 
smallatndel = 0.2;          % 0.05 * 40 
zerocarb = 0.02;

% Start with OC gone. This will switch 'on' during the analysis.
%  (Remember that the loop starts with the end of the analysis.)
ocgone = true;
ocmaxtemp = 700;            % max temp where OC evolves in ox mode (C)
oczerocarb = 0.1;           % max OC release in last step of He mode
                            % to consider that OC has been removed

% Set the initial flags. ocgone is allowed to change later in the loop, but
% presently doesn't.
osout.atnloss = osin.cATN>smallatndel;
osout.ocgone = (osout.mode==oxmode) & osout.temp>ocmaxtemp;

% OC has died out to zero by end of He-- assume OC is always gone
if osout.FID(find(osout.mode==oxmode, 1)-1) < oczerocarb
   osout.ocgone(find(osout.mode)==oxmode) = 1;
end

% Set to zero if it will be treated as zero
osout.cATN(find(~osout.atnloss & osout.cATN>0))=0;

% Storage array for k-sigma
osout.ks = zeros(osin.N, SpeciesN);

% Start the x-vector with zeroes
x = zeros(SpeciesN, 1);

% Start the state vector. x is the amount of material in the reactor. 
% xhist is the state vector at each point in time.
xhist = [];
dxhist = [];

% ---- Analysis step loop -----------------------------------
for n = osin.N:-1:1

   % Get the matrix of attenuation coefficients. This could have been
   % done before the loop, but it is being left flexible
   osout.ks(n,:) = GetAtn(osin.temp(n), osin.tau(n), ksguess, opticsz);

   % Turn OC on if inert atmosphere
%   if ocgone & osin.mode(n)==1
%       ocgone = false;
%   end
%   osout.ocgone(n) = ocgone;

   % This is the initial R (Equation xx in paper). 
   [R, stepy] = Step_Assumption(osout, n);
   
   % Constraint: pc & lac must have same sign. 
   if ocgone & sign(x(pc)) ~= sign(x(lac)) & x(pc)>0.01 & x(lac)>0.01
%       ocgone = false;
   end
  
   % Estimate dx. The left divide will return an answer even if 
   %   the system is over/underdetermined. Because the program deals with
   %   the underdetermined case next, this isn't really necessary, 
   %   except to prevent the underdetermined case from giving an error.
   dx = R\stepy;           

   % Handle underdetermined system
   osout.underdet(n) = (size(stepy, 1)<size(R,2));

   % Call the underdetermined program with reduced row echelon form of 
   %  state equations
   if osout.underdet(n)

       % Get underdetermined solution
       [rrange, osout.underdet_err(n)] = Step_Underdet(rref([R stepy]));   
       osout.solfound(n) = ~isempty(rrange);

       % Range found: use weights to find point along line
       if osout.solfound(n)
          dx = rrange * uweight;
       
       % Too small to worry about
       elseif max(abs(dx)) < zerocarb
          osout.solfound(n) = true;     % turn off warning
           
       end
       
   % not underdetermined; flag solution
   else
       osout.solfound(n) = 1;
       osout.underdet_err(n) = 0;
   end
   
   % Get the new value of x (state vector) 
   x = x + dx;

   % Add x to the history
   xhist = [x'; xhist];
   dxhist = [dx'; dxhist];
end

osout.x = xhist;
osout.dx = dxhist;
return


% --- Find the analysis time steps ----------------------------------------
function [chunks, newfrac] = GetTimeSteps(tp, fracid, lumplen)
   
% Loop through the time steps 
chunks = [];
newfrac = [];

for i=1:size(tp, 1)
   perdlen = tp(i,2) - tp(i,1) + 1;
    
    % the first step is shortest
   step1 = mod(perdlen, lumplen);
   endstep = (tp(i, 1)+step1 - 1);
   if step1>0
      chunks = [chunks; [tp(i, 1) endstep]];
   end
   
   nsteps = (perdlen-step1)/lumplen;
   for j=1:nsteps
      chunks = [chunks; [endstep+1 endstep+lumplen]];
      endstep = endstep+lumplen;
   end
   newfrac = [newfrac; ones([(nsteps+1) 1])*fracid(i)];
end

return

% ----- Summarize the 1-second data based on the temperature program in tp 
% Returns a structure containing summed/averaged data (depending on the 
% field)
function aos = MakeSumData(os, tp, fracid)
global SpeciesN

   aos = struct();
   aos.N = size(tp,1);
   
   % create means
   for i=1:aos.N
       idx = (tp(i, 1)):(tp(i, 2));
       aos.time(i) = mean(os.time(idx));
       aos.len(i) = length(idx);        % a-step length
       aos.fracid(i) = fracid(i);       % Fraction belongs to
       aos.FID(i) = sum(os.FID(idx));   % FID is total released
       aos.cATN(i) = sum(os.ATN(idx));  % ATN is total released (use correct)
       
       % Rest of variables are averages
       aos.temp(i) = mean(os.temp(idx));
       aos.mode(i) = mean(os.mode(idx));
       aos.laser(i) = mean(os.laser(idx));
       aos.blaser(i) = mean(os.blaser(idx));       
   end

   % turn into column vectors
   fnames = fieldnames(aos);
   for i=2:length(fnames)
       aos.(fnames{i}) = aos.(fnames{i})';
   end
   
   % add info
   aos.tau = aos.laser ./ aos.blaser;
   zeromat = zeros([aos.N SpeciesN]);
   aos.ks = zeromat;
   aos.x = zeromat;
   aos.dx = zeromat;
   
   % add flag data
   zmat_log = logical(zeros([aos.N 1]));
   
   flag_descrip = {'OC gone?', 'ATN loss?', 'Underdetermined?', 'Solution found?' 'PC/LAC mismatch?'};
   flagname = {'ocgone', 'atnloss', 'underdet', 'solfound', 'mismatch'};
   
   % set all to zero
   for i=1:length(flagname)
      aos.(flagname{i}) = zmat_log;
   end
   aos.underdet_err = zeromat;
   
   aos.flagname = flagname;
   aos.flag_descrip = flag_descrip;
   
return

% ----- Summarize the values in each fraction ----
function fsum = FracSum(aos)
global SpeciesN

   fsum = zeros([max(aos.fracid) SpeciesN]);   
   
   % Loop through analysis steps & add to the appropriate fraction
   for i=1:aos.N
       fracno = aos.fracid(i);
       if fracno>0
          fsum(fracno, :) = fsum(fracno, :) + aos.dx(i,:);
       end
   end

return

% ----- Sign vector ------------------------
function signvec = GetSign(vec, val0)
   signvec = sign(vec);
   signvec(find(abs(vec) <= val0)) = 0;
return
   