% thermabsgram(struc) creates a thermogram from a TOA data structure (see
% ReadOCEC)
%
% Optional arguments: 
%     'ksigma'     ksigma for absorption trace (default: 'pc'; ['lac', 'off'])
%     'absrat'     array containing multipliers for additional absorption traces
%     'lasernorm'  normalizing value for laser signal (default 1000)
%     'splits'     identify split point(s) [ (on) | horiz | vert | off]
%     'showcal'    show calibration period (default false)
%     'legendpos'  legend position (default Best)
%     'scale'      value for scaling FID and ATN (default 1)
%     'color'      thermogram in color? (default true)
%     'showtotal'  show total EC and OC (default true)
function [carb] = thermabsgram(os, varargin)

% Author: Tami Bond, University of Illinois, yark@uiuc.edu
% Version 2.1   29 Nov 2007
% This program is distributed under the Creative Commons by-sa license 3.0
%   (http://creativecommons.org/licenses/by-sa/3.0/).
%
% Required additional files:
%   GetAtn (advanced users may want to modify), Unify
%   varg_val, varg_proof (utilities)

% check arguments
if ~varg_proof(varargin, {'ksigma', 'absrat', 'showcal', 'lasernorm',...
        'splits', 'scale', 'color', 'showtotal'}, true)
    return
end
global of

% Set values from user input, or use defaults 
showabs = varg_val(varargin, 'ksigma', 'pc');
absrat = varg_val(varargin, 'absrat', []);
showcal = varg_val(varargin, 'showcal', false);
lasernorm = varg_val(varargin, 'lasernorm', 100000);
splittype = varg_val(varargin, 'splits', 'on');
scalefac = varg_val(varargin, 'scale', 1);
iscolor = varg_val(varargin, 'color', true);
hastotal = varg_val(varargin, 'showtotal', true);

% determine value of absorbing carbon to use
if isnumeric(showabs)
    ksigma = showabs;
else
    switch showabs
        case 'pc'
            ksigma = 45;                 % guess of PC absorption
      case 'lac'
            ksigma = 6.8*2.3;            % guess of LAC absorption
        case 'off'
            ksigma = 0;
        otherwise
            ksigma = 0;
    end
end

% Normalizing values
tempnorm = ceil(max(os.temp)/100)*100;
absnorm = ksigma;                       % ksigma gets m2 to g
hasrefl = isfield(os, 'refl');          % check if has reflectance

% Make vectors that include or don't include calibration data
if showcal
    of = Unify(os);
else
    of = os;
end

% Axis positions etc
maxtime = max(of.time);
ax3room = 1.15;                          % increase to make room for 3rd axis
legendroom = 1;                       % fractional decrease to make room for legend

% Determine colors {FID, temp, laser, abs, absrat, refl, splitpt, modebreak}
if iscolor
    colr = {'b', 'r', 'k', 'g', 'g:', 'k:', 'rx', 'b:'};
else
    colr = {'--k', ':k', 'k', '-.k', '-.k', 'k', 'kx', 'k:'};
end
fsize = 13;         % Font size

% Text to add to carbon scaling
if scalefac~=1
    carbtx = ['*' num2str(scalefac)];
else
    carbtx = '';
end

% Start figure and set position
figure;
pos = get(gcf, 'Position');
pos(3) = pos(3) * 1.3;
set(gcf, 'Position', pos);

% Squeeze to make room for 2nd axis
ax1 =gca;
pos = get(ax1, 'Position');
pos(3) = pos(3)/ax3room;
pos(4) = pos(4)/legendroom;
set(ax1, 'XLim', [0 maxtime], 'Position', pos,...
    'FontSize', fsize);
hold on;

% FID trace
ln=plot(of.time, of.FID * scalefac, colr{1});
lg{1} = ['Carbon' carbtx];

% absorption plot
if ksigma>0
    plot(ax1, of.time, of.ATN/absnorm * scalefac, colr{4});
    lg{length(lg)+1} = ['AbsC' carbtx ' (ksigma=' num2str(ksigma) ')'];
end

% other absorption plots
for i=1:size(absrat)
    plot(ax1, of.time, absrat(i)*of.ATN/absnorm * scalefac, colr{5});
    lg{length(lg)+1} = ['ATN' carbtx '*' num2str(absrat(i))];
end
get(gca, 'YLim');

% y-limit for scaling
lylim = get(gca, 'YLim');
YMAX = lylim(2);
lylim(2) = lylim(2)*1.05;

% Laser transmittance trace
plot(of.time, of.laser./of.blaser * YMAX, colr{3});
lg{length(lg)+1} = 'Transmittance';

% reflectance trace
if hasrefl
    plot(of.time, of.refl/lasernorm, colr{6});
    lg{length(lg)+1} = 'Reflectance';
end

% Temperature plot
plot(of.time, of.temp/tempnorm*YMAX, colr{2});
lg{length(lg)+1} = 'Temp';
set(gca, 'YLim', lylim);

ylabel('Carbon release (ug/cm2/sec)');
xlabel('Analysis time (sec)');
set(get(gca, 'Children'), 'LineWidth', 1.5);

% Put legend on
lgh = legend(lg, 'Orientation', 'horizontal', 'Location', 'NorthOutside');
set(lgh, 'FontSize', fsize-1);
pos = get(gca, 'Position');

% New axis for different temperature scale
ax2=axes;
set(ax2, 'YLim', lylim/YMAX*tempnorm, 'YAxisLocation', 'right',...
    'XLim', [0 maxtime], 'Color', 'none', 'XTick', [],...
    'Position', pos, 'FontSize', fsize);
yl=ylabel('Temperature (C)');

% New axis for laser scale
ax3=axes;
pos3 = [pos(1)+pos(3)*ax3room pos(2) pos(3)*0.001 pos(4)];

set(ax3, 'YLim', lylim/YMAX, 'YAxisLocation', 'right',...
    'Color', 'none', 'XTick', [],...
    'FontSize', fsize, 'Position', pos3);
ylabel('Laser (I/I0)');

% Back to first axes
axes(ax1);

% ---------- handle splits ------------------------------
% show mode split
oxmodetime = find(of.mode>1,1);
plot([oxmodetime oxmodetime], lylim, colr{8});

if strcmp(splittype, 'on')
   % put transmittance split on plot
   ln = plot(ax1, os.split_tr, of.laser(os.split_tr)/of.blaser(os.split_tr)*YMAX, colr{7});
   set(ln, 'MarkerSize', 10);

   % put reflectance split on plot
   if hasrefl
      ln = plot(ax1, os.split_rf, os.refl(os.split_rf)/lasernorm, colr{7});
      set(ln, 'MarkerSize', 10);
   end
end

% Generate title
titletext = [os.name ' ' os.info ];
if scalefac ~=1
    titletext = [titletext ' (x' num2str(scalefac) ')'];
end
tl = title(titletext);
set(tl, 'Units', 'normalized', 'Interpreter', 'none');
tlpos = get(tl, 'Position');
tlpos(2) = tlpos(2) * 1.15;
set(tl, 'Position', tlpos);

hold off;

% Calculate EC/OC (the Sunset way)
nlen = length(os.FID);
addtag = '';

ECt = sum(os.FID((os.split_tr+1):nlen));
if hasrefl
   ECr = sum(os.FID((os.split_rf+1):nlen));
   addtag = 't';
end

% Calculate total
TC = sum(os.FID);
carb = [ECt (TC-ECt)];
return
